'
' This module tests low level X-10 transmission using a TW-523 or PL-513.
'
'     TW-523  Adapter   Function
'       1     yellow    Z-C collector
'       2     green     Common
'       3      red      Rx Envelope collector (not used)
'       4     black     Tx Envelope Anode
'
'     PL-513  Adapter   Function
'       1     yellow    Z-C collector
'       2     green     Z-C emitter
'       3      red      Tx Envelope cathode
'       4     black     Tx Envelope anode
'
' The zero-crossing signal must be connected to INT0.
'
#if Option.TargetDevice = "ZX128e"
Const txPin as Byte = 36
Const syncPin as Byte = 18
#elseif Option.TargetDevice = "ZX1281" Or Option.TargetDevice = "ZX1280"
Const txPin as Byte = B.1
Const syncPin as Byte = B.7
#elseif Option.TargetDevice = "ZX128a1"
Const txPin as Byte = B.1
Const syncPin as Byte = B.7
#elseif Option.TargetDevice = "ZX40a"
Const txPin as Byte = C.1
Const syncPin as Byte = C.0
#else
#define HAS_LEDS
Const txPin as Byte = 15
Const syncPin as Byte = 13
#endif

Dim oq(1 to 40) as Byte

' bit stream for "House Code A, Device 1"
Dim hcA_dev1 as ByteVectorData ({ &He6, &H96, &H97, &H9a, &H5a, &H50 })

' bit stream for "House Code A, Command On"
Dim hcA_on as ByteVectorData ({ &He6, &H95, &H9b, &H9a, &H5c, &H60 })

' bit stream for "House Code A, Command Off"
Dim hcA_off as ByteVectorData ({ &He6, &H95, &Hab, &H9a, &H56, &Ha0 })

Sub Main()
	Dim i as Integer
	Call PutPin(syncPin, 1)
	Call PutPin(A.0, 0)
	Call DefineX10(1, 0, txPin, &H00)
	Call OpenQueue(oq)
	Call OpenX10(1, 0, oq)
	i = 0
	Do
#if defined(HAS_LEDS)
		Call PutPin(pin.redLED, LoByte(i) And 1)
#endif
		Call addX10Data(hcA_dev1.DataAddress, SizeOf(hcA_dev1))
		If (CBool(LoByte(i) And 1)) Then
			Call addX10Data(hcA_on.DataAddress, SizeOf(hcA_on))
		Else
			Call addX10Data(hcA_off.DataAddress, SizeOf(hcA_off))
		End If
		Call Delay(1.0)
		i = i + 1
'		Debug.Print "Pass "; i
	Loop
End Sub

Sub addX10Data(ByVal pmAddr as Long, ByVal cnt as Integer)
	Dim data as ProgMem Byte based pmAddr
	Do While (cnt > 0)
		Dim b as Byte
		b = data
		Call PutPin(syncPin, 0)
		Call PutQueue(oq, b, 1)
		pmAddr = pmAddr + 1
		cnt = cnt - 1
	Loop
End Sub
