'
' This module tests the low level X10 reception using a TW-523.
'
'     TW-523  Adapter   Function
'       1     yellow    Z-C collector
'       2     green     Common
'       3      red      Rx Envelope collector
'       4     black     Tx Envelope Anode (not used)
'
' The zero-crossing signal must be connected to INT0.
'
#if Option.TargetDevice = "ZX128e"
Const syncPin as Byte = 18
#elseif Option.TargetDevice = "ZX1281" Or Option.TargetDevice = "ZX1280"
Const syncPin as Byte = B.7
#elseif Option.TargetDevice = "ZX128a1"
Const syncPin as Byte = C.0
Const rxPin as Byte = C.3
#elseif Option.TargetDevice = "ZX40a"
Const syncPin as Byte = C.7
#else
#define HAS_LEDS
Const rxPin as Byte = 14
Const syncPin as Byte = 13
#endif

Dim iq(1 to 40) as Byte

' This data array is used to translate bit-encoded house code values
' to the corresponding house code.  The index at which the raw value
' is found corresponds to the house code as shown in the comments.
Dim houseCodes as ByteVectorData({
	&H69	' 0110 1001  A
	&Ha9	' 1010 1001  B
	&H59	' 0101 1001  C
	&H99	' 1001 1001  D
	&H56	' 0101 0110  E
	&H96	' 1001 0110  F
	&H66	' 0110 0110  G
	&Ha6	' 1010 0110  H
	&H6a	' 0110 1010  I
	&Haa	' 1010 1010  J
	&H5a	' 0101 1010  K
	&H9a	' 1001 1010  L
	&H55	' 0101 0101  M
	&H95	' 1001 0101  N
	&H65	' 0110 0101  O
	&Ha5	' 1010 0101  P
})

' This data array is used to translate bit-encoded command code values
' to the corresponding command code.  The index at which the raw value
' is found corresponds to the command code as shown in the comments.
Dim commandCodes as IntegerVectorData({
	&H01a5	' 01 1010 0101   1
	&H02a5	' 10 1010 0101   2
	&H0165	' 01 0110 0101   3
	&H0265	' 10 0110 0101   4
	&H0159	' 01 0101 1001   5
	&H0259	' 10 0101 1001   6
	&H0199	' 01 1001 1001   7
	&H0299	' 10 1001 1001   8
	&H01a9	' 01 1010 1001   9
	&H02a9	' 10 1010 1001  10
	&H0169	' 01 0110 1001  11
	&H0269	' 10 0110 1001  12
	&H0155	' 01 0101 0101  13
	&H0255	' 10 0101 0101  14
	&H0195	' 01 1001 0101  15
	&H0295	' 10 1001 0101  16
	&H0156	' 01 0101 0110  All Units Off
	&H015a	' 01 0101 1010  All Lights On
	&H0166	' 01 0110 0110  On
	&H016a	' 01 0110 1010  Off
	&H0196	' 01 1001 0110  Dim
	&H019a	' 01 1001 1010  Bright
	&H01a6	' 01 1010 0110  All Lights Off
	&H01aa	' 01 1010 1010  Extended Code
	&H0256	' 10 0101 0110  Hail Request
	&H025a	' 10 0101 1010  Hail Ack
	&H0266	' 10 0110 0110  Pre-set Dim 0
	&H026a	' 10 0110 1010  Pre-set Dim 1
	&H0296	' 10 1001 0110  Ext. Analog Data
	&H029a	' 10 1001 1010  Status=On
	&H02a6	' 10 1010 0110  Status=Off
	&H02aa	' 10 1010 1010  Status Request
})

' This table is used to produce the command name for command codes
' above 16.
Dim cmdCodeStrings as StringVectorData({
	"All Units Off"
	"All Lights On"
	"On"
	"Off"
	"Dim"
	"Bright"
	"All Lights Off"
	"Extended Code"
	"Hail Request"
	"Hail Ack"
	"Pre-set Dim 0"
	"Pre-set Dim 1"
	"Ext. Analog Data"
	"Status=On"
	"Status=Off"
	"Status Request"
})

Sub Main()
	Dim i as Integer
Call PutPin(C.7, 0)
	Call PutPin(syncPin, 1)
	Call DefineX10(1, rxPin, 0, &H01)
	Call OpenQueue(iq)
	Call OpenX10(1, iq, 0)
	i = 0
	Do
		If (GetQueueCount(iq) >= 1) Then
			Dim b as Byte
			' look for a start mark
			Call PeekQueue(iq, b, 1)
			If ((b And &Hfc) <> &H38) Then
				' discard the byte
				Call GetQueue(iq, b, 1)
'Debug.Print "Discarding byte 0x"; CStrHex(b)
			ElseIf (GetQueueCount(iq) >= 3) Then
				Dim houseCode as String
				Dim cmdCode as String
				Dim data(1 to 3) as Byte
	
				Call GetQueue(iq, data, 3)

'Debug.Print "0x"; CStrHex(data(1));
'Debug.Print " 0x"; CStrHex(data(2));
'Debug.Print " 0x"; CStrHex(data(3))

				' determine the house code
				Dim houseCodeVal as Byte
				Dim validCode as Boolean
				houseCodeVal = Shl(data(1), 6) Or Shr(data(2), 2)
'Debug.Print "raw house code is 0x"; CStrHex(houseCodeVal)
				validCode = False
				For i = 1 to UBound(houseCodes)
					If (houseCodes(i) = houseCodeVal) Then
						houseCode = "House code: " & Chr(i + &H40)
						validCode = True
						Exit For
					End If
				Next i
				If (Not validCode) Then
					houseCode = "House code: ? [0x" & CStrHex(houseCodeVal) & "]"
				End If
	
				' determine the command code
				Dim cmdCodeVal as Integer
				cmdCodeVal = CInt(MakeWord(data(3), data(2) And &H03))
'Debug.Print "raw command code is 0x"; CStrHex(cmdCodeVal)
				validCode = False
				For i = 1 to UBound(commandCodes)
					If (commandCodes(i) = cmdCodeVal) Then
						If (i <= 16) Then
							cmdCode = "Unit: " & CStr(i)
							i = 0
						Else
							cmdCode = "Command: " & cmdCodeStrings(i - 16)
						End If
						validCode = True
						Exit For
					End If
				Next i
				If (Not validCode) Then
					cmdCode = "Command: ? [0x" & CStrHex(cmdCodeVal) & "]"
				End If
	
				' display the result
				Debug.Print houseCode; ", "; cmdCode
			End If
		End If
	Loop
End Sub
